local _G = _G
--local AddOn, ns = ...

local Round3DPortraits = LibStub("AceAddon-3.0"):NewAddon("Round3DPortraits", "AceEvent-3.0")

_G.Round3DPortraits = Round3DPortraits

-- Localization stuff, but I'm a lazy bastard, so good luck.
--local L = ns.L

local MyConsole = LibStub("AceConsole-3.0")

-- I should be setting this to false for release of the AddOn
local useUglyHack = false

--local haveDonePlayerRotation = false
--local haveDoneTargetRotation = false

local PlayerR3DFrameScripts = {}
local TargetR3DFrameScripts = {}
local R3DFrameScripts = {}
local R3DFrame = {}
local PlayerR3DFrame = {}

-- Utility functions for the options database
local getOpt, setOpt, pb4InUse

do
   function getOpt(info)
      local key = info[#info]
      --MyConsole:Print(key, "=", Round3DPortraits.db.profile[key])
      return Round3DPortraits.db.profile[key]
   end

   function setOpt(info, value)
      local key = info[#info]
      Round3DPortraits.db.profile[key] = value
      Round3DPortraits:Refresh()
   end

   function pb4InUse(info)
      if Round3DPortraits.db.profile.usePitbull then
	 return true
      end
      return false
   end
end


Round3DPortraits.options = {
   type = "group",
   args = {
      global = {
	 type = "group",
	 name = "Global Settings",
	 order = 1,
	 args = {
	    __header1 = {
	       type = "description",
	       name = "Options for setting portrait frame characteristics.",
	       order = 1,
	    },
	    __caveat = {
	       type = "description",
	       name = "Note: if you toggle the 'Use Pitbull4' option the frames become immobile.  To move them just drag the Pitbull4 frames instead.",
	       order = 2,
	    },
	    show = {
	       name = "Show",
	       type = "toggle",
	       desc = "Show or hide the frames",
	       set = setOpt,
	       get = getOpt,
	       width = "half",
	       order = 10,
	    },
	    lockMovement = {
	       name = "Lock Frames",
	       type = "toggle",
	       desc = "Lock frames in position",
	       set = setOpt,
	       get = getOpt,
	       order = 11,
	       disabled = pb4InUse,
	    },
	    usePitbull = {
	       name = "Use PitBull4",
	       type = "toggle",
	       desc = "Use PitBull4 unit frames as parent of Round3DPortrait frames.  Meaningless without PitBull4 installed.",
	       set = setOpt,
	       get = getOpt,
	       order = 12,
	    },
	    setEliteTextures = {
	       name = "Fancy Elites",
	       type = "toggle",
	       desc = "Draw elite and rare dragons around appropriate portraits",
	       set = setOpt,
	       get = getOpt,
	       --width = "half",
	       order = 13,
	    },
	    --showBarTex = {
	    --   name = "Bar Textures",
	    --   type = "toggle",
	    --   desc = "Draw health/power bar overlay texture",
	    --   set = setOpt,
	    --   get = getOpt, 
	    --   order = 14,
	    --},
	    doRoundBorder = {
	       name = "Round all edges",
	       type = "toggle",
	       desc = "Draw border rounded on all sides of portrait, good for a portrait independent of any unit frame",
	       set = setOpt,
	       get = getOpt,
	       order = 15,
	    },
	    isClamped = {
	       name = "Clamp to screen",
	       type = "toggle",
	       desc = "Clamp frames to screen",
	       set = setOpt,
	       get = getOpt,
	       order = 16,
	    },
	    dimensionsHeading = {
	       type = "header",
	       name = "Scale and placement",
	    },
	    scale = {
	       name = "Scale",
	       type = "range",
	       desc = "Scale of player and target frames",
	       min = 0.1, max = 3, step = 0.005,
	       get = getOpt,
	       set = setOpt,
	       order = 101,
	    },
	    shieldScale = {
	       name = "Level button scale",
	       type = "range",
	       desc = "Scale of level button on frames",
	       min = 0.4, max = 2, step = 0.05,
	       get = getOpt,
	       set = setOpt,
	       order = 102,
	    },
	    playerX = {
	       name = "Player x",
	       type = "range",
	       desc = "Offset from center of screen in the horizontal direction",
	       min = -840, max = 840, softMin = -200, softMax = 200, step = 0.5,
	       get = getOpt,
	       set = setOpt,
	       order = 103,
	       disabled = pb4InUse,
	    },
	    playerY = {
	       name = "Player y",
	       type = "range",
	       desc = "Offset from center of screen in the vertical direction",
	       min = -840, max = 840, softMin = -200, softMax = 200, step=0.5,
	       get = getOpt,
	       set = setOpt,
	       order = 104,
	       disabled = pb4InUse,
	    },
	    targetX = {
	       name = "Target x",
	       type = "range",
	       desc = "Offset from center of screen in the horizontal direction",
	       min = -840, max = 840, softMin = -200, softMax = 200, step=0.5,
	       get = getOpt,
	       set = setOpt,
	       order = 105,
	       disabled = pb4InUse,
	    },
	    targetY = {
	       name = "Target y",
	       type = "range",
	       desc = "Offset from center of screen in the vertical direction",
	       min = -840, max = 840, softMin = -200, softMax = 200, step=0.5,
	       get = getOpt,
	       set = setOpt,
	       order = 106,
	       disabled = pb4InUse,
	    },	    
	 },
      },
   },
}

local defaults = {
   profile = {
      setEliteTextures = true,
      show = true,
      lockMovement = false,
      usePitbull = false,
      showBarTex = false,
      doRoundBorder = true,
      isClamped = true,
      scale = 1.0,
      shieldScale = 0.9,
      playerX = -200,
      playerY = -150,
      targetX = 200,
      targetY = -150,
   },
}

function Round3DPortraits:OnInitialize()
   self.db = LibStub("AceDB-3.0"):New("Round3DPortraitsDB", defaults, true)
   self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
   self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
   self.db.RegisterCallback(self, "OnProfileReset", "Refresh")
   self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
   self.options.args.profiles.order = 100
end

function Round3DPortraits:OnEnable()
   LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Round3DPortraits", 
							 self.options, 
							 nil)
   local ACD = LibStub("AceConfigDialog-3.0")
   self.OptionsPanel = ACD:AddToBlizOptions(self.name, self.name, nil, "global")
   self.OptionsPanel.Profiles = ACD:AddToBlizOptions(self.name, "Profiles", 
						     self.name, "profiles")

   -- Register events
   Round3DPortraits:RegisterEvent("UNIT_MODEL_CHANGED")
   Round3DPortraits:RegisterEvent("PLAYER_TARGET_CHANGED")
   Round3DPortraits:RegisterEvent("PLAYER_ENTERING_WORLD")
   Round3DPortraits:RegisterEvent("PLAYER_UPDATE_RESTING")
   Round3DPortraits:RegisterEvent("PLAYER_REGEN_ENABLED")
   Round3DPortraits:RegisterEvent("PLAYER_REGEN_DISABLED")
   Round3DPortraits:RegisterEvent("PLAYER_LEVEL_UP")

   -- Create frames
   --  First the base frames, player as a SecureUnitButton
   local frameDimension = 64
   Round3DPortraits.PlayerBaseFrame = CreateFrame("Button", 
						  "PlayerRound3DPortrait", 
						  UIParent,
						  "SecureUnitButtonTemplate")
   local play = Round3DPortraits.PlayerBaseFrame
   Round3DPortraits.TargetBaseFrame = CreateFrame("Button",
						  "TargetRound3DPortrait",
						  UIParent)
   local tar = Round3DPortraits.TargetBaseFrame
   play:SetWidth(frameDimension)
   play:SetHeight(frameDimension)
   play:SetFrameLevel(10)
   tar:SetWidth(frameDimension)
   tar:SetHeight(frameDimension)
   tar:SetFrameLevel(10)
   play:Hide()
   tar:Hide()

   --  Now the child frames for holding artwork and other layers
   --  Also create the texture and fontstring objects
   play.bg = play:CreateTexture("PlayerR3DBG", "BACKGROUND")
   tar.bg = tar:CreateTexture("TargetR3DBG", "BACKGROUND")
   play.bg:SetWidth(frameDimension)
   play.bg:SetHeight(frameDimension)
   tar.bg:SetWidth(frameDimension)
   tar.bg:SetHeight(frameDimension)

   -- Player and target 3D models
   play.model = CreateFrame("PlayerModel", "PlayerR3DModel", play)
   tar.model = CreateFrame("PlayerModel", "TargetR3DModel", tar)
   play.model:EnableMouse(false)
   tar.model:EnableMouse(false)
   play.model:SetWidth(frameDimension*0.75)
   play.model:SetHeight(frameDimension*0.75)
   tar.model:SetWidth(frameDimension*0.75)
   tar.model:SetHeight(frameDimension*0.75)
   play.guid = UnitGUID("player")
   --tar.guid = UnitGUID("target")

   -- Borders behind and in front of model
   play.hub = play.model:CreateTexture("PlayerR3DHub", "ARTWORK")
   tar.hub = tar.model:CreateTexture("TargetR3DHub", "ARTWORK")

   --play.portrait2d = play.model:CreateTexture("PlayerR3DFallback", "OVERLAY")
   tar.portrait2d = tar.model:CreateTexture("TargetR3DFallback", "OVERLAY")

   play.prettyFrame = CreateFrame("Button", "PlayerR3DPretty", play)
   tar.prettyFrame = CreateFrame("Button", "TargetR3DPretty", tar)
   play.prettyFrame:EnableMouse(false)
   tar.prettyFrame:EnableMouse(false)
   play.prettyFrame:SetFrameLevel(play.model:GetFrameLevel() + 1)
   tar.prettyFrame:SetFrameLevel(tar.model:GetFrameLevel() + 1)
   play.pretty = play.prettyFrame:CreateTexture("PlayerR3DBorder", "OVERLAY")
   tar.pretty = tar.prettyFrame:CreateTexture("TargetR3DBorder", "OVERLAY")
   play.hub:SetWidth(frameDimension)
   play.hub:SetHeight(frameDimension)
   --play.portrait2d:SetWidth(frameDimension)
   --play.portrait2d:SetHeight(frameDimension)
   tar.hub:SetWidth(frameDimension)
   tar.hub:SetHeight(frameDimension)
   tar.portrait2d:SetWidth(frameDimension)
   tar.portrait2d:SetHeight(frameDimension)
   play.pretty:SetWidth(frameDimension*2)
   play.pretty:SetHeight(frameDimension*2)
   tar.pretty:SetWidth(frameDimension*2)
   tar.pretty:SetHeight(frameDimension*2)

   -- Texture to hold level text and boss skull
   play.badgeFrame = CreateFrame("Button", "PlayerR3DBadgeFrame", play.prettyFrame)
   tar.badgeFrame = CreateFrame("Button", "TargetR3DBadgeFrame", tar.prettyFrame)
   play.badgeFrame:EnableMouse(false)
   tar.badgeFrame:EnableMouse(false)
   play.badgeFrame:SetPoint("CENTER", play, "CENTER", 0, 0)
   tar.badgeFrame:SetPoint("CENTER", tar, "CENTER", 0, 0)

   play.badge = play.badgeFrame:CreateTexture("PlayerR3DBadge", "OVERLAY")
   tar.badge = tar.badgeFrame:CreateTexture("TargetR3DBadge", "OVERLAY")

   -- FontString for level text
   play.fontFrame = CreateFrame("Button", "PlayerR3DLevelFontFrame", play.badgeFrame)
   tar.fontFrame = CreateFrame("Button", "TargetR3DLevelFontFrame", tar.badgeFrame)
   play.fontFrame:EnableMouse(false)
   tar.fontFrame:EnableMouse(false)
   play.fontFrame:SetPoint("CENTER", play, "CENTER", 0, 0)
   tar.fontFrame:SetPoint("CENTER", tar, "CENTER", 0, 0)

   play.levelstr = play.fontFrame:CreateFontString("PlayerLevelString", "OVERLAY")
   tar.levelstr = tar.fontFrame:CreateFontString("TargetLevelString", "OVERLAY")
   play.levelstr:SetFont("Fonts\\FRIZQT__.TTF", 8)
   tar.levelstr:SetFont("Fonts\\FRIZQT__.TTF", 8)

   -- Overlay for level text to contain resting icon or boss skull
   play.shieldFrame = CreateFrame("Button", "PlayerR3DShieldFrame", play.fontFrame)
   tar.shieldFrame = CreateFrame("Button", "TargetR3DShieldFrame", tar.fontFrame)
   play.shieldFrame:EnableMouse(false)
   tar.shieldFrame:EnableMouse(false)

   play.shield = play.shieldFrame:CreateTexture("PlayerR3DShield", "OVERLAY")
   tar.shield = tar.shieldFrame:CreateTexture("TargetR3DShield", "OVERLAY")
   play.shield:SetWidth(28)
   play.shield:SetHeight(32)
   tar.shield:SetWidth(18)
   tar.shield:SetHeight(18)

   -- Combat icon, replaces level text when in combat
   play.combatIcFrame = CreateFrame("Button", "PlayerR3DCombatIconFrame", play.fontFrame)
   play.combatIcFrame:EnableMouse(false)

   play.combatIc = play.combatIcFrame:CreateTexture("PlayerR3DCombatIcon", "OVERLAY")
   play.combatIc:SetWidth(23)
   play.combatIc:SetHeight(23)

   for key, val in pairs(R3DFrameScripts) do
      --print("hooking", val)
      play:HookScript(key, val)
      tar:SetScript(key, val)
      --print("gone tomorrow")
   end
   for key, val in pairs(PlayerR3DFrameScripts) do
      play:HookScript(key, val)
   end
   for key, val in pairs(TargetR3DFrameScripts) do
      tar:SetScript(key, val)
   end
   for key, val in pairs(PlayerR3DFrame) do
      --print(key, val)
      play[key] = val
   end
   for key, val in pairs(R3DFrame) do
      play[key] = val
   end

   -- Not sure why this isn't working yet, will debug
   local isClamped = Round3DPortraits.db.profile.isClamped
   if isClamped then
      play:SetClampedToScreen(true)
      tar:SetClampedToScreen(true)
   else
      play:SetClampedToScreen(false)
      tar:SetClampedToScreen(false)
   end

   play:SetMovable(true)
   play:RegisterForDrag("LeftButton")
   play:RegisterForClicks("AnyUp")
   play:SetAttribute("type1", "target")
   play:SetAttribute("type2", "menu")

   tar:SetMovable(true)
   tar:RegisterForDrag("LeftButton")
   tar:RegisterForClicks("AnyUp", "AnyDown")

end

function Round3DPortraits:DrawPlayerPortrait()
   local frame = Round3DPortraits.PlayerBaseFrame
   local show = Round3DPortraits.db.profile.show
   local xPos = Round3DPortraits.db.profile.playerX
   local yPos = Round3DPortraits.db.profile.playerY
   local scale = Round3DPortraits.db.profile.scale
   local shieldScale = Round3DPortraits.db.profile.shieldScale
   local doRoundBorder = Round3DPortraits.db.profile.doRoundBorder
   local usePitbull = Round3DPortraits.db.profile.usePitbull
   local PB4 = PitBull4
   --if PitBull4 then print("Found PitBull4") end
   local levformatstr, r, g, b, level = Round3DPortraits:MakeLevelText("player")

   -- Set all frame positions, textures, and formatted text
   frame:SetScale(scale)
   if PB4 and usePitbull then
      frame:ClearAllPoints()
      frame:SetPoint("RIGHT", PitBull4_Frames_player, "LEFT", 1.1, -0.5)
   else
      frame:ClearAllPoints()
      frame:SetPoint("CENTER", nil, "CENTER", xPos, yPos)
   end

   frame.badge:SetSize(shieldScale*32, shieldScale*32)
   frame.shield:SetSize(shieldScale*28, shieldScale*32)
   frame.combatIc:SetSize(shieldScale*23, shieldScale*23)

   frame.bg:SetTexture("Interface\\AddOns\\Round3DPortraits\\Textures\\ModelBack")
   frame.bg:SetPoint("CENTER", frame, "CENTER", 0, 0)
   frame.bg:SetVertexColor(1, 1, 1, 1)

   frame.model:SetPoint("CENTER", frame, "CENTER", 0, -1)

   frame.hub:SetTexture("Interface\\AddOns\\Round3DPortraits\\Textures\\CircleBorder")
   frame.hub:SetPoint("CENTER", frame, "CENTER", 0, 0)

   if not doRoundBorder then
      frame.pretty:SetTexture("Interface\\AddOns\\Round3DPortraits\\Textures\\BlizzPortBorder")
      frame.pretty:SetPoint("CENTER", frame, "CENTER", -4.5, -1.5)
   else
      frame.pretty:SetTexture("Interface\\AddOns\\Round3DPortraits\\Textures\\BlizzPortBorderFullRound")
      frame.pretty:SetPoint("CENTER", frame, "CENTER", -4.5, -1.5)
   end

   frame.badge:SetTexture("Interface\\AddOns\\Round3DPortraits\\Textures\\LevelBadgePlayer")
   frame.badge:SetPoint("CENTER", frame, "CENTER", -22, -21.5)
   --frame.badge:SetRotation(-math.pi/2)
   --if not haveDonePlayerRotation then
   --   frame.badge:SetSize(1.2*frame.badge:GetWidth(),
--			  1.2*frame.badge:GetHeight())
   --end
   --haveDonePlayerRotation = true

   frame.levelstr:SetFormattedText(levformatstr, 240, 240, 210, level)
   frame.levelstr:SetPoint("CENTER", frame, "CENTER", -21.5, -20.5)

   frame.shield:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
   frame.shield:SetTexCoord(0., 0.5, 0., 0.499)
   frame.shield:SetPoint("CENTER", frame, "CENTER", -21.5, -22.5)

   frame.combatIc:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
   frame.combatIc:SetTexCoord(0.5, 1., 0., 0.499)
   frame.combatIc:SetPoint("CENTER", frame, "CENTER", -20.75, -20)

   if ( not IsResting() ) then
      frame.shield:Hide()
   end

   if (not InCombatLockdown() ) then
      frame.combatIc:Hide()
   else
      frame.shield:Hide()
      frame.combatIc:Show()
   end

   if show then
      frame:Show()
   else
      frame:Hide()
   end

   --frame:SetFrameLevel(10)

   if show then
      frame:SetAttribute("unit", "player")
      RegisterUnitWatch(frame)
   else
      frame:SetAttribute("unit", nil)
      UnregisterUnitWatch(frame)
      frame:Hide()
   end
   frame.model:SetUnit("player")
   frame.model:SetCamera(0)
end

function Round3DPortraits:DrawTargetPortrait()
   local frame = Round3DPortraits.TargetBaseFrame
   local show = Round3DPortraits.db.profile.show
   local xPos = Round3DPortraits.db.profile.targetX
   local yPos = Round3DPortraits.db.profile.targetY
   local scale = Round3DPortraits.db.profile.scale
   local shieldScale = Round3DPortraits.db.profile.shieldScale
   local setelite = Round3DPortraits.db.profile.setEliteTextures
   local doRoundBorder = Round3DPortraits.db.profile.doRoundBorder
   local levformatstr, r, g, b, level = Round3DPortraits:MakeLevelText("target")
   local elitestat = UnitClassification("target")
   local usePitbull = Round3DPortraits.db.profile.usePitbull
   local PB4 = PitBull4

   frame:SetScale(scale)

   local uiScale = UIParent:GetEffectiveScale()

   if PB4 and usePitbull then
      frame:ClearAllPoints()
      frame:SetPoint("LEFT", PitBull4_Frames_target, "RIGHT", -1.5, -1.75)
   else
      frame:ClearAllPoints()
      frame:SetPoint("CENTER", nil, "CENTER", xPos, yPos)
   end

   frame.badge:SetSize(shieldScale*32, shieldScale*32)
   frame.shield:SetSize(shieldScale*18, shieldScale*18)

   frame.bg:SetTexture("Interface\\AddOns\\Round3DPortraits\\Textures\\ModelBack")
   frame.bg:SetPoint("CENTER", frame, "CENTER", 0.5, 0)
   frame.bg:SetVertexColor(1, 1, 1, 1)

   frame.model:SetPoint("CENTER", frame, "CENTER", 0.5, -1)

   frame.hub:SetTexture("Interface\\AddOns\\Round3DPortraits\\Textures\\CircleBorder")
   frame.hub:SetPoint("CENTER", frame, "CENTER", 0.5, 0)

   local fileName, borderX, borderY
   if setelite and (elitestat == "elite" or (level == 99 and not UnitIsPlayer("target")) and setelite) then
      if not doRoundBorder then
	 fileName = "Interface\\AddOns\\Round3DPortraits\\Textures\\BlizzPortBorderElite"
      else
	 fileName = "Interface\\AddOns\\Round3DPortraits\\Textures\\BlizzPortBorderEliteFullRound"
      end
      borderX = 16.5
      borderY = -6.5
   elseif elitestat == "rare" and setelite then
      if not doRoundBorder then
	 fileName = "Interface\\AddOns\\Round3DPortraits\\Textures\\BlizzPortBorderRare"
      else
	 fileName = "Interface\\AddOns\\Round3DPortraits\\Textures\\BlizzPortBorderRareFullRound"
      end
      borderX = 16.5
      borderY = -6.5
   else
      if not doRoundBorder then
	 fileName = "Interface\\AddOns\\Round3DPortraits\\Textures\\BlizzPortBorderRight"
	 borderX = 4.5
	 borderY = -1.5
      else
	 fileName = "Interface\\AddOns\\Round3DPortraits\\Textures\\BlizzPortBorderFullRound"
	 borderX = -3.5
	 borderY = -1.5
      end
   end

   --MyConsole:Print(fileName)
   frame.pretty:SetTexture(fileName)
   frame.pretty:SetPoint("CENTER", frame, "CENTER", borderX, borderY)

   frame.badge:SetTexture("Interface\\AddOns\\Round3DPortraits\\Textures\\LevelBadgeTarget")
   frame.badge:SetPoint("CENTER", frame, "CENTER", 22, -21.5)

   frame.levelstr:SetFormattedText(levformatstr, r, g, b, level)
   frame.levelstr:SetPoint("CENTER", frame, "CENTER", 21.5, -20.5)

   if ( level == 99 ) then
      frame.shield:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Skull")
      frame.levelstr:Hide()
   else
      frame.shield:SetTexture(0, 0, 0, 0)
      frame.levelstr:Show()
   end
   frame.shield:SetPoint("CENTER", frame, "CENTER", 22.5, -20.5)

   if show then
      frame:Show()
   else
      frame:Hide()
   end

   frame.portrait2d:SetPoint("CENTER", frame, "CENTER", 0, 0)

   if UnitIsPlayer("target") 
   and (UnitInParty("target") or UnitInRaid("target"))
   and not UnitInRange("target") then
      frame.model:ClearModel()
      SetPortraitTexture(frame.portrait2d, "target")
      frame.portrait2d:Show()
   else
      frame.model:SetUnit("target")
      frame.portrait2d:Hide()
   end
   frame.model:SetCamera(0)
end



-- Event handlers
function Round3DPortraits:UNIT_MODEL_CHANGED(junk, unitId)
   --MyConsole:Print("model")
   Round3DPortraits:Refresh()
end

function Round3DPortraits:PLAYER_TARGET_CHANGED()
   if ( UnitExists("target") ) then
      Round3DPortraits:DrawTargetPortrait()
   else
      Round3DPortraits.TargetBaseFrame:Hide()
   end
end

function Round3DPortraits:PLAYER_ENTERING_WORLD()
   Round3DPortraits:DrawPlayerPortrait()

   -- Bad, bad hack
   -- Go home, bad hack
   if useUglyHack then
      BuffFrame:SetScale(1.2)
      if ConsolidatedBuffs then
	 ConsolidatedBuffs:SetScale(1.2)
      end
   end

end

function Round3DPortraits:PLAYER_UPDATE_RESTING()
   if IsResting() then
      Round3DPortraits.PlayerBaseFrame.shield:Show()
   else
      Round3DPortraits.PlayerBaseFrame.shield:Hide()
   end
end

function Round3DPortraits:PLAYER_REGEN_DISABLED()
   local frame = Round3DPortraits.PlayerBaseFrame
   frame.combatIc:Show()
   frame.levelstr:Hide()
   frame.shield:Hide()
   --self.PlayerR3DFrame:PLAYER_REGEN_DISABLED()
   if frameThatsMoving then
      R3DFrameScripts.OnDragStop(frameThatsMoving)
   end
end
function Round3DPortraits:PLAYER_REGEN_ENABLED()
   local frame = Round3DPortraits.PlayerBaseFrame
   frame.combatIc:Hide()
   frame.levelstr:Show()
   if IsResting() then
      frame.shield:Show()
   else
      frame.shield:Hide()
   end
   --self:PLAYER_REGEN_ENABLED_MINE()
end

function Round3DPortraits:PLAYER_LEVEL_UP()
   -- At this point, a redraw is the only thing that seems to update the level text
   --local levformatstr, r, g, b, level = Round3DPortraits:MakeLevelText("player")
   --Round3DPortraits.PlayerBaseFrame.levelstr:SetFormattedText(levformatstr, 240, 240, 210, level)
   Round3DPortraits:Refresh()
end

function Round3DPortraits:OnDisable()
   MyConsole:Print("Disable Round3DPortraits")
end

function Round3DPortraits:Refresh()
   --MyConsole:Print("Refresh")
   Round3DPortraits:DrawPlayerPortrait()
   if (UnitExists("target")) then
      Round3DPortraits:DrawTargetPortrait()
   end

   if Round3DPortraits.db.profile["usePitbull"] and not Round3DPortraits.db.profile.lockMovement then
      Round3DPortraits.db.profile["lockMovement"] = true
      LibStub("AceConfigRegistry-3.0"):NotifyChange("Round3DPortraits")
   end
end

-- Utility
function Round3DPortraits:MakeLevelText(theUnit)
   local level = UnitLevel(theUnit)
   if level <= 0 then
      level = 99
   end
   local color = GetQuestDifficultyColor(level)
   return "|cff%02x%02x%02x%s", color.r*255, color.g*255, color.b*255, level
end

local frameThatsMoving = nil
function R3DFrameScripts:OnDragStart()
   local db = Round3DPortraits.db.profile
   --local frame = Round3DPortraits.PlayerBaseFrame
   if db.lockMovement or InCombatLockdown() then
      return
   end

   self:StartMoving()
   frameThatsMoving = self
end

function R3DFrameScripts:OnDragStop()
   if frameThatsMoving ~= self then return end
   frameThatsMoving = nil
   self:StopMovingOrSizing()

   -- I'm still having a hard time getting the repositioning right when syncing
   --  the position to the db after dragging the frame.
   local uiScale = UIParent:GetEffectiveScale()
   local scale = self:GetEffectiveScale() / uiScale
   local x, y = self:GetCenter()
   x, y = x*scale, y*scale
   x = x - GetScreenWidth()/2
   y = y - GetScreenHeight()/2

   if self == Round3DPortraits.PlayerBaseFrame then
      Round3DPortraits.db.profile["playerX"] = x/self:GetScale()
      Round3DPortraits.db.profile["playerY"] = y/self:GetScale()
      --print(Round3DPortraits.db.profile["playerX"], 
	--    Round3DPortraits.db.profile["playerY"])
   elseif self == Round3DPortraits.TargetBaseFrame then
      Round3DPortraits.db.profile["targetX"] = x/self:GetScale()
      Round3DPortraits.db.profile["targetY"] = y/self:GetScale()
   end

   LibStub("AceConfigRegistry-3.0"):NotifyChange("Round3DPortraits")

   Round3DPortraits:Refresh()
end

function R3DFrameScripts:OnMouseUp(button)
   if button == "LeftButton" then
      return R3DFrameScripts.OnDragStop(self)
   end
end

function PlayerR3DFrame:PLAYER_REGEN_DISABLED()
   print("")
end

function PlayerR3DFrame:PLAYER_REGEN_ENABLED()
   --frame.combatIc:Hide()
   --frame.levelstr:Show()
   --if IsResting() then
   --   frame.shield:Show()
   --else
   --   frame.shield:Hide()
   --end
   print("In function")
end

function PlayerR3DFrameScripts:OnEnter()
   if InCombatLockdown() then return end
   GameTooltip_SetDefaultAnchor(GameTooltip, self)
   GameTooltip:SetUnit("player")
   local r, g, b = GameTooltip_UnitColor("player")
   GameTooltipTextLeft1:SetTextColor(r, g, b)
   GameTooltip:Show()
   --Round3DPortraits:PositionTest()
end

function PlayerR3DFrameScripts:OnLeave()
   GameTooltip:Hide()
end

function TargetR3DFrameScripts:OnEnter()
   if InCombatLockdown() then return end
   GameTooltip_SetDefaultAnchor(GameTooltip, self)
   GameTooltip:SetUnit("target")
   local r, g, b = GameTooltip_UnitColor("target")
   GameTooltipTextLeft1:SetTextColor(r, g, b)
   GameTooltip:Show()
end

function TargetR3DFrameScripts:OnClick(button, isDown)
   --MyConsole:Print("OnClick", self, button, down)
   if button == "RightButton" and not isDown and not InCombatLockdown() then
      R3DFrame:menu("target")
   end
end

function TargetR3DFrameScripts:OnLeave()
   GameTooltip:Hide()
end


--function R3DFrameScripts:OnShow()
--   Round3DPortraits:Refresh()
--end

local Round3DPortraitsDropDown = CreateFrame("Frame", "Round3DPortraitsDropDown", UIParent, "UIDropDownMenuTemplate")
UnitPopupFrames[#UnitPopupFrames+1] = "Round3DPortraitsDropDown"

local function getMenuFromUnit(unit)
   
   if UnitIsUnit(unit, "player") then
      return "SELF"
   end
   
   -- I hope to implement vehicle switching at some point, not a huge priority
   if UnitIsUnit(unit, "vehicle") then
      return "VEHICLE"
   end
   
   if not UnitIsPlayer(unit) then
      return "TARGET"
   end

   return "PLAYER"
end

local menuUnit = nil
local function menuInitFunc()
   if not menuUnit then return end
   local menu = getMenuFromUnit(menuUnit)
   if menu then
      UnitPopup_ShowMenu(Round3DPortraitsDropDown, menu, menuUnit, nil, nil)
   end
end
UIDropDownMenu_Initialize(Round3DPortraitsDropDown, menuInitFunc, "MENU", nil)

function R3DFrame:menu(unit)
   menuUnit = unit
   ToggleDropDownMenu(1, nil, Round3DPortraitsDropDown, "cursor")
end


-- This function was for debugging, in here for reference
--  Creates a basic unit frame to test dragging, targeting, menu, etc.
function Round3DPortraits:FrameTest()
   Round3DPortraits.PlayerInteractionFrame = CreateFrame("Button", "PlayerR3DInteractionFrame", Round3DPortraits.PlayerBaseFrame, "SecureUnitButtonTemplate")
   local frame = Round3DPortraits.PlayerInteractionFrame
   frame:SetPoint("CENTER", nil, "CENTER", -200, -100)
   --frame:SetPoint("CENTER", Round3DPortraits.PlayerBaseFrame, "CENTER", 0, 0)
   frame:SetWidth(64)
   frame:SetHeight(64)
   frame:SetFrameLevel(20)

   --frame.texture = frame:CreateTexture("FrameTestTex", "BACKGROUND")
   --frame.texture:SetTexture("Interface\\Icons\\ACHIEVEMENT_BOSS_KINGYMIRON_01")
   --frame.texture:SetAllPoints()

   frame:SetMovable(true)
   frame:RegisterForDrag("LeftButton")
   frame:RegisterForClicks("AnyUp")
   frame:SetAttribute("*type1", "target")
   frame:SetAttribute("*type2", "menu")
   frame:SetAttribute("unit", "player")

   for key, val in pairs(R3DFrameScripts) do
      frame:HookScript(key, val)
   end
   for key, val in pairs(PlayerR3DFrameScripts) do
      frame:HookScript(key, val)
   end
   for key, val in pairs(PlayerR3DFrame) do
      frame[key] = val
   end
   for key, val in pairs(R3DFrame) do
      frame[key] = val
   end

   frame:SetAttribute("unit", "player")
   RegisterUnitWatch(frame)   

end

-- Another debugging function, still here but nothing calls it
function Round3DPortraits:PositionTest()
   local playX = Round3DPortraits.db.profile.playerX
   local playY = Round3DPortraits.db.profile.playerY
   local frame = Round3DPortraits.PlayerBaseFrame
   local uiScale = UIParent:GetEffectiveScale()
   local scale = frame:GetEffectiveScale() / uiScale
   local baseScale = frame:GetScale()
   local x, y = frame:GetCenter()
   MyConsole:Print("----")
   MyConsole:Print("DEBUG:", "UI Scale:", UIParent:GetEffectiveScale())
   MyConsole:Print("DEBUG:", "Frame eff scale:", frame:GetEffectiveScale())
   MyConsole:Print("DEBUG:", "Frame scale:", baseScale, scale)
   MyConsole:Print("DEBUG:", "Saved coords:", playX, playY)
   MyConsole:Print("DEBUG:", "GetCenter coords:", x, y)
   MyConsole:Print("DEBUG:", "Moved to center:", x-GetScreenWidth()/2, y-GetScreenHeight()/2)
   MyConsole:Print("DEBUG:", "Screen resolution", GetScreenWidth(), GetScreenHeight())
   x, y = x*scale, y*scale
   x = x - GetScreenWidth() / 2
   y = y - GetScreenHeight() / 2
   MyConsole:Print("DEBUG:", "Scaled:", x, y)
end